import 'package:flutter/services.dart';
import 'package:tingyun_flutter_plugin/src/agent/http/dns_info.dart';
import 'package:tingyun_flutter_plugin/src/agent/interface/web_request_timing.dart';
import 'package:tingyun_flutter_plugin/src/agent/util/logger.dart';
import 'package:tingyun_flutter_plugin/src/agent/util/string_utils.dart';

class TingyunWebRequestTimingImpl implements TYWebRequestTiming {
  final MethodChannel _methodChannel;
  final String _requestTag;
  final String _url;

  String _ipAddress = "";
  int _dnsTime = 0;
  String _hostName = "";

  TingyunWebRequestTimingImpl(this._requestTag, this._url, this._methodChannel){
    try {
      _hostName = Uri.parse(this._url).host;
    } catch (e) {
      Logger().d("error get hostName :${e.toString()}");
    }
  }

  @override
  void start() {
    if (_requestTag != null &&
        !StringUtils.isStringNullEmptyOrWhitespace(_url)) {
      _methodChannel.invokeMethod("startWebRequestTiming", <String, dynamic>{
        "requestTag": _requestTag,
        "url": _url,
        "host": _hostName,
        "startTime": DateTime.now().millisecondsSinceEpoch,
      });
    } else {
      Logger().d("Web Request Timing could not be created!",
          logType: LogType.Error);
    }
  }

  @override
  void stop(int responseCode) {
    // if (_requestTag != null &&
    //     !StringUtils.isStringNullEmptyOrWhitespace(_url) &&
    //     responseCode != null &&
    //     !StringUtils.isStringNullEmptyOrWhitespace(responseMessage)) {

    int dnsDura = DnsInfo.getDnsDuration(_hostName);
    String ip = DnsInfo.getIpAddress(_hostName);
    withDnsInfo(dnsDura, ip);

    if (_requestTag != null &&
        !StringUtils.isStringNullEmptyOrWhitespace(_url) &&
        responseCode != null) {
      int endTime = DateTime.now().millisecondsSinceEpoch;
      if(this._dnsTime != null && this._dnsTime > 0){
        endTime += this._dnsTime;
      }

      _methodChannel.invokeMethod("stopWebRequestTiming", <String, dynamic>{
        "requestTag": _requestTag,
        "url": _url,
        "dns": this._dnsTime,
        "ip": this._ipAddress,
        "endTime": endTime,
        "responseCode": responseCode,
        // "responseMessage": responseMessage
      });
    } else {
      // Logger().d(
      //     "Web Request Timing could not be stopped!" +
      //         "- Request Tag: $_requestTag Url: $_url responseCode: $responseCode responseMessage: $responseMessage",
      //     logType: LogType.Error);
      Logger().d(
          "Web Request Timing could not be stopped!" +
              "- Request Tag: $_requestTag Url: $_url responseCode: $responseCode",
          logType: LogType.Error);
    }
  }

  @override
  String getRequestTag() {
    return _requestTag;
  }

  @override
  String getRequestTagHeader() {
    return "X-Tingyun";
  }

  @override
  void withDnsInfo(int dnsTime, String ipInfo) {
    if(dnsTime != null) {
      //dnsTime小于或者等于这个阈值，算是复用dns吧，所以就不设置
      if(dnsTime >= 2) {
        this._dnsTime = dnsTime;
      }
    }
    if(ipInfo != null) {
      this._ipAddress = ipInfo;
    }
    return;
  }


}