// You have generated a new plugin project without
// specifying the `--platforms` flag. A plugin project supports no platforms is generated.
// To add platforms, run `flutter create -t plugin --platforms <platforms> .` under the same
// directory. You can also find a detailed instruction on how to add platforms in the `pubspec.yaml` at https://flutter.dev/docs/development/packages-and-plugins/developing-packages#plugin-platforms.

import 'dart:async';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/action.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/interface/root_action.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/null_actions/null_root_action_impl.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/web_request_timing_impl.dart';
import 'package:tingyun_flutter_plugin/src/agent/http/dns_info.dart';
import 'package:tingyun_flutter_plugin/src/agent/interface/newlens_agent.dart';
import 'package:tingyun_flutter_plugin/src/agent/util/logger.dart';
import 'package:tingyun_flutter_plugin/src/agent/util/string_utils.dart';

import '../../tingyun_flutter_plugin.dart';
import 'action/counter.dart';
import 'action/null_web_request_timing.dart';
import 'http/http_instrumentation.dart';
import 'model/configuration.dart';
import 'model/log_level.dart';

class TingyunFlutterImpl implements Tingyun{
  static const String methodChannelName = "tingyun_flutter_plugin/newlens";

  static Tingyun? _instance;
  final MethodChannel _methodChannel;
  final Function _startupUpFunction;

  static String currentRoutePage = "";

  bool _started = false;
  bool _enabled = false;
  bool _crashReporting = true;
  bool _actionReporting = false;
  bool _uiReporting = false;
  bool _networkEnabled = false;
  TingyunRootAction _pageMonitor = TingyunNullRootAction();

  /**
   * 设置一个page结束时间的阈值
   */
  int _pageLoadingTime = 500;

  int _networkDelayTime = 2000;

  factory TingyunFlutterImpl(){
    if(_instance == null){
      final MethodChannel _methodChannel =
          const MethodChannel(methodChannelName);
      _instance = TingyunFlutterImpl.private(_methodChannel);
    }
    return _instance as TingyunFlutterImpl;
  }

  ///named construct
  TingyunFlutterImpl.private(this._methodChannel,
      {Function startupFunction = runApp})
      : _startupUpFunction = startupFunction;

  Future<void> start(Widget widget, {Function? onError, Function? flutterOnError, int? pageLoadingTime, int? networkDelay}) async{
    Logger().i("tingyun flutter impl start");

    await runZoned(() async{
      await _start(widget, pageLoadingTime: pageLoadingTime, networkDelay: networkDelay, flutterOnError: flutterOnError);
    }, onError: (error, stacktrace) async{
      if (onError != null) {
        if (onError is void Function(Object, StackTrace)) {
          onError(error, stacktrace);
        } else {
          Logger().i('onError Must be Function(Object, StackTrace)');
        }
      }

      Logger().d("runZoned onError");
      // if (_crashReporting && error.runtimeType != null) {
      if (_actionReporting && error.runtimeType != null) {
        // Logger().d("errorName:${error.runtimeType.toString()}, "
        //     "reason:${error.message.toString()}, stack:${stacktrace.toString()}");
        Logger().d("error:${error.runtimeType} stack:${stacktrace.toString()}");
        // await reportCrash(error.runtimeType.toString(),
        //     error.message.toString(), stacktrace.toString(), type: 2);

        String reason = "";
        try{
          reason = error.message.toString();
        }catch(e){
          // Logger().d("error message not exist:${e.toString()}");
          reason = error.toString();
        }

        await _reportCrash(error.runtimeType.toString(),
            reason, stacktrace.toString(), type: 2);

      }
    });
  }

  Future<void> _start(Widget topLevelWidget,
      { int? pageLoadingTime = 0, int? networkDelay = 0,
        Configuration configuration = const Configuration(), Function? flutterOnError}) async{
    try {
      ///调用这个方法，因为我们在runApp之前有调用methodChannel的操作
      WidgetsFlutterBinding.ensureInitialized();

      ///这里获取了native层的一些配置内容
      Map<String, bool>? autoStartConfig = await getAutoStratConfiguration();
      Logger().i("get configure from tingyun:${autoStartConfig}");

      if (!_isPluginStarted()) {
        _started = true;

        if(pageLoadingTime != null && pageLoadingTime > 0 )
          this._pageLoadingTime = pageLoadingTime;

        if(networkDelay != null && networkDelay > 0 )
          this._networkDelayTime = networkDelay;

        if (autoStartConfig!['enabled']!) {
          _enabled = true;
          // Take over the log level of native agent
          configuration = Configuration(
              logLevel:
                  autoStartConfig["logLevel"]! ? LogLevel.Debug : LogLevel.Info,
              reportCrash: autoStartConfig["crashReporting"]!,
              monitorWebRequest: autoStartConfig["webRequest"]!,
              uiEnabled: autoStartConfig["ui"]!,
              userActionEnabled: autoStartConfig["action"]!,
          );
        } else {
          Logger().d('autoStart is false, because enabled is false, please check');
          throw('enabled is false, please check');
          Logger().d('eeeeeee');
          // if (configuration.isAutoStartPropertyAvailable) {
            // We need to do a manual startup if it is available
            // 调用native层的逻辑进行相关的start,这里我们自己的不需要，
            // 我们手动的在native调用一下我们初始化的接口就行了
            // _methodChannel.invokeMethod(
            //     "start", configuration.getStartupConfiguration());
          // }
        }

        Logger().logLevel = configuration.logLevel;

        if (configuration.isWebRequestMonitoringEnabled) {
          // Override the HTTP communication with our handler
          /// 这里进行相关的系统方面的内容的处理
          HttpOverrides.global = TingyunHttpOverrides(_instance!);
        } else {
          Logger().d("Webrequest Instrumentation deactivated.");
        }

        _networkEnabled = configuration.isWebRequestMonitoringEnabled;
        _crashReporting = configuration.isCrashReportingEnabled;
        _uiReporting = configuration.uiEnabled;
        _actionReporting = configuration.actionEnabled;

        if (!configuration.isCrashReportingEnabled) {
          Logger().d("Crash reporting deactivated.", logType: LogType.Info);
        }

        FlutterError.onError = (FlutterErrorDetails details) async {

          if (flutterOnError != null) {
            if (flutterOnError is void Function(FlutterErrorDetails details)) {
              flutterOnError(details);
            } else {
              Logger().i('flutterOnError Must be Function(FlutterErrorDetails details)');
            }
          }

          Logger().d('FlutterError.onError happened');
          FlutterError.dumpErrorToConsole(details, forceReport: true);
          // if (_crashReporting && details.exception.runtimeType != null) {
          if (_actionReporting && details.exception.runtimeType != null) {
            String errorName = details.exception.runtimeType.toString();
            String reason = details.toStringShort() != null
                ? "${details.exceptionAsString()} - ${details.toStringShort()}"
                    .substring(errorName.length + 2)
                : details.exceptionAsString();

            await _reportCrash(errorName, reason, details.stack.toString(), type: 2);
          }
        };
      }

      Logger().i("tingyun flutter plugin success!");
    } catch (e) {
      Logger().i(
          "tingyun flutter plugin failed to initialize. Error: ${e.toString()}",
          logType: LogType.Error);
      //开关设置成false
      _crashReporting = false;
      _networkEnabled = false;
      _uiReporting = false;
      _actionReporting = false;

      _started = false;
    }

    //调用原来的启动的函数
    _startupUpFunction(topLevelWidget);
  }

  Future<Map<String, bool>?> getAutoStratConfiguration() async{
    return await _methodChannel.invokeMapMethod("getAutoStartConfiguration");
  }

  bool _isEnabled(){
    return _started && _enabled;
  }

  bool _isPluginStarted(){
    return _started;
  }

  @visibleForTesting
  set started(bool isStarted){
    this._started = isStarted;
  }

  /// 1:定义错误,2:捕获异常
  // @override
  Future<void> _reportCrash(String errorName, String reason, String stacktrace,
      {int type = 1}) async {
    if(!_isEnabled()){
      Logger().d("Newlens().start() method was not called!",
          logType: LogType.Warning);
      return;
    }

    Logger().d('begin do crash report');
    if (!StringUtils.isStringNullOrEmpty(errorName)) {
      // if (_crashReporting) {
      if(_actionReporting){
        Logger().d('begin invoke method channel reportCrash');
        // await _methodChannel.invokeMethod("reportCrash", <String, dynamic>{
        //   "errorValue": errorName,
        //   "reason": reason,
        //   "stacktrace": stacktrace,
        //   "type": type
        // });

        await _methodChannel.invokeMethod("reportError", <String, dynamic>{
          "errorValue": errorName,
          "reason": reason,
          "stacktrace": stacktrace,
          "type": type,
          "props": Map<String, dynamic>()
        });
      }
    } else {
      Logger().d(
          "Crash will not be reported because something is missing! errorName:$errorName reason:$reason stacktrace: $stacktrace",
          logType: LogType.Error);
    }
  }

  Future<Map<String, String>?> getHTTPTagForWebRequest(String url) async {
    return await _methodChannel
        .invokeMapMethod("getRequestTagForInterceptor", <String, dynamic>{"url": url});
  }

  // Future<int> startWebRequestTiming(String requestTag, String url) async {
  //   if (!StringUtils.isStringNullEmptyOrWhitespace(requestTag) &&
  //       !StringUtils.isStringNullEmptyOrWhitespace(url)) {
  //     return await _methodChannel.invokeMethod("startWebRequestTiming",
  //         <String, dynamic>{"requestTag": requestTag, "url": url});
  //   } else {
  //     Logger().d(
  //         "Web Request Timing could not be created! Url: $url requestTag: $requestTag",
  //         logType: LogType.Error);
  //   }
  //
  //   return -1;
  // }
  //
  // Future<bool> stopWebRequestTiming(int webTimingKey, String url,
  //     int responseCode, String responseMessage) async {
  //   if (webTimingKey != null &&
  //       webTimingKey != -1 &&
  //       !StringUtils.isStringNullEmptyOrWhitespace(url) &&
  //       responseCode != null &&
  //       !StringUtils.isStringNullEmptyOrWhitespace(responseMessage)) {
  //     return await _methodChannel
  //         .invokeMethod("stopWebRequestTiming", <String, dynamic>{
  //       "webTimingKey": webTimingKey,
  //       "url": url,
  //       "responseCode": responseCode,
  //       "responseMessage": responseMessage
  //     });
  //   } else {
  //     Logger().d(
  //         "Web Request Timing could not be stopped!" +
  //             "- WebTimingKey: $webTimingKey Url: $url responseCode: $responseCode responseMessage: $responseMessage",
  //         logType: LogType.Error);
  //   }
  //
  //   return false;
  // }

  @override
  TingyunRootAction enterAction(String name, {int type = 0}) {
    if (!_isEnabled()) {
      Logger().d(
          "Tingyun().start() method was not called! Action will have no effect.",
          logType: LogType.Warning);
      return TingyunNullRootAction();
    }

    if (StringUtils.isStringNullOrEmpty(name)) {
      Logger().d("Action Name was empty or null! Action will have no effect.",
          logType: LogType.Error);
      return TingyunNullRootAction();
    }

    if(!_uiReporting) {
      Logger().d("Action reporting is false.",
          logType: LogType.Warning);
      return TingyunNullRootAction();
    }

    int key = ActionCounter().getNewActionId();
    _methodChannel.invokeMethod("enterAction", <String, dynamic>{
      "name": "flutterPage $currentRoutePage#$name",
      "key": key,
      "type": type,
      "enterTime": DateTime.now().millisecondsSinceEpoch.toString(),
    });

    TingyunRootAction result = TingyunRootActionImpl.private(key, _methodChannel);
    /// 设置一个回调，如果过了一定时间，rootAction还没有leave的话，就自动的结束掉，以免对数据产生干扰
    Future.delayed(Duration(milliseconds: 19000), (){
    // Future.delayed(Duration(milliseconds: 10000), (){
      if(!result.isClosed()){
        Logger().d("over time not invoke leaveAction ,so do it myself");
        result.leaveAction();
      }
    });

    return result;
  }

  @override
  void endSession() {
    // if (_isEnabled()) {
    //   _methodChannel.invokeMethod("endVisit", <String, dynamic>{
    //     "platform": platform != null ? platform.index : null
    //   });
    // } else {
    //   Logger().i("Tingyun().start() method was not called!",
    //       logType: LogType.Warning);
    // }
  }

  @override
  void setUserIdentifier(String user) {
    if (!_isEnabled()) {
      Logger().d("Tingyun().start() method was not called!",
          logType: LogType.Warning);
      return;
    }

    _methodChannel.invokeMethod("setUserIdentifier", <String, dynamic>{
      "user": user
    });
  }

  @override
  void flushEvents() {
    // if (_isEnabled()) {
    //   _methodChannel.invokeMethod("flushEvents", <String, dynamic>{
    //     "platform": platform != null ? platform.index : null
    //   });
    // } else {
    //   Logger().i("Tingyun().start() method was not called!",
    //       logType: LogType.Warning);
    // }
  }

  // @override
  // void reportError(String errorName, int errorCode, {Platform platform}) {
  //   if (!_isEnabled()) {
  //     Logger().i("Tingyun().start() method was not called!",
  //         logType: LogType.Warning);
  //     return;
  //   }
  //
  //   if (!StringUtils.isStringNullOrEmpty(errorName) && errorCode != null) {
  //     _methodChannel.invokeMethod("reportError", <String, dynamic>{
  //       "errorName": errorName,
  //       "errorCode": errorCode,
  //       "platform": platform != null ? platform.index : null
  //     });
  //   } else {
  //     Logger().d(
  //         "Error was not reported! ErrorName:$errorName errorCode:$errorCode",
  //         logType: LogType.Error);
  //   }
  // }
  @override
  void reportError(String message, String stackTrace, {Map<String, dynamic>? props}) {
    if (!_isEnabled()) {
      Logger().d("Tingyun().start() method was not called!",
          logType: LogType.Warning);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(message) &&
        !StringUtils.isStringNullOrEmpty(stackTrace)) {
      // if (_crashReporting) {
      if(_actionReporting){
        _methodChannel.invokeMethod("reportError", <String, dynamic>{
          "reason": message,
          "stacktrace": stackTrace,
          "type": 1,
          "props": props
        });
      }
    } else {
      Logger().d(
          "Error was not reported! ErrorName:$message errorCode:$stackTrace",
          logType: LogType.Error);
    }
  }

  @override
  Future<TYWebRequestTiming> createWebRequest(String url) async {
    if (!_isEnabled()) {
      Logger().d("Tingyun().start() method was not called!",
          logType: LogType.Warning);
      return NullWebRequestTiming();
    }

    if(!_networkEnabled) {
      Logger().d("_networkEnabled is false.",
          logType: LogType.Warning);
      return NullWebRequestTiming();
    }

    String tag = await _methodChannel.invokeMethod(
        "getRequestTag");
    //设置dns时间
    try {
      DnsInfo.setDnsInfo(Uri.parse(url).host);
    } catch (e) {
      Logger().d("error setDnsInfo :${e.toString()}");
    }

    return createWebRequestTiming(tag, url);
  }

  /// requestTag相当于做了网络的一个唯一的标识的id
  TYWebRequestTiming createWebRequestTiming(String requestTag, String url) {
    if (!_isEnabled()) {
      Logger().d("Tingyun().start() method was not called!",
          logType: LogType.Warning);

      return NullWebRequestTiming();
    }

    return TingyunWebRequestTimingImpl(requestTag, url, _methodChannel);
  }

  @override
  int getPageLoadingDuration() {
    return this._pageLoadingTime;
  }

  @override
  void leaveBreadcrumb(String breadCrumb) {
    if (!_isEnabled()) {
      Logger().d("Tingyun().start() method was not called!",
          logType: LogType.Warning);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(breadCrumb)) {
      _methodChannel.invokeMethod("leaveBreadcrumb", <String, dynamic>{
        "breadCrumb": breadCrumb
      });
    } else {
      Logger().d(
          "leaveBreadcrumb was not reported! breadCrumb:$breadCrumb",
          logType: LogType.Error);
    }
  }

  @override
  void onEvent(String eventId, {String eventTag = "", Map<String, dynamic>? props}) {
    if (!_isEnabled()) {
      Logger().d("Tingyun().start() method was not called!",
          logType: LogType.Warning);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(eventId)) {
      _methodChannel.invokeMethod("onEvent", <String, dynamic>{
        "eventId": eventId,
        "tag": eventTag,
        "props": props
      });
    } else {
      Logger().d(
          "onEvent was not reported! eventId:$eventId",
          logType: LogType.Error);
    }
  }

  @override
  void setUserCrashMessage(String key, String value) {
    if (!_isEnabled()) {
      Logger().d("Tingyun().start() method was not called!",
          logType: LogType.Warning);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(key) && !StringUtils.isStringNullOrEmpty(value)) {
      _methodChannel.invokeMethod("setUserCrashMessage", <String, dynamic>{
        "key": key,
        "value": value
      });
    } else {
      Logger().d(
          "setUserCrashMessage was not reported! key:$key, value:$value",
          logType: LogType.Error);
    }
  }

  @override
  int getNetworkDelay() {
    return this._networkDelayTime;
  }

  @override
  void setCustomOnResumeEndIns(){
    if (!_isEnabled()) {
      Logger().d("Tingyun().setCustomOnResumeEndIns() method was not called!",
          logType: LogType.Warning);
      return;
    }

    _methodChannel.invokeMethod("setCustomOnResumeEndIns", <String, dynamic>{
      "className": ""
    });
  }

  @override
  void startPage({String? pageName}) {
    if (!_isEnabled()) {
      Logger().d(
          "Tingyun().start() method was not called! Action will have no effect.",
          logType: LogType.Warning);
      return ;
    }

    if(!_uiReporting) {
      Logger().d("Action reporting is false.",
          logType: LogType.Warning);
      return;
    }

    //page没有关的话，就关一下
    if(!_pageMonitor.isClosed()) {
      Logger().d("_pageMonitor close old unclosed minitor.",
          logType: LogType.Warning);
      _pageMonitor.leaveAction();
    }

    if(!StringUtils.isStringNullOrEmpty(pageName)) {
      currentRoutePage = pageName!;
    }
    _pageMonitor = this.enterAction("$currentRoutePage", type: 1);
  }

  @override
  void endPage() {
    if (!_isEnabled()) {
      Logger().d(
          "Tingyun().start() method was not called! Action will have no effect.",
          logType: LogType.Warning);
      return ;
    }

    if(!_uiReporting) {
      Logger().d("Action reporting is false.",
          logType: LogType.Warning);
      return;
    }

    if(_pageMonitor.isClosed()) {
      Logger().d("_pageMonitor has been closed, so return",
          logType: LogType.Warning);
      return;
    }

    _pageMonitor.leaveAction(props: <String, dynamic>{
      "networkDelay": this.getNetworkDelay()
    }, type: 1);
  }

}