import 'package:flutter/widgets.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/interface/root_action.dart';
import 'package:tingyun_flutter_plugin/src/agent/agent_impl.dart';
import 'package:tingyun_flutter_plugin/src/agent/interface/web_request_timing.dart';
import 'package:tingyun_flutter_plugin/src/agent/model/configuration.dart';

abstract class Tingyun{

  factory Tingyun() => TingyunFlutterImpl();

  /// Start function which should be called to start the agent.The [topLevelWidget]
  /// is the widget which is starting the application. The Configuration contains
  /// default values for all properties. reportCrash value is true per default.
  /// monitorWebRequest is true per default.
  Future<void> start(Widget widget, {Function onError, Function flutterOnError, int pageLoadingTime, int networkDelay});

  /// Creating a [TingyunRootAction] which is able to have child actions. If
  /// you enter [null] or an empty [String] for the [name] you will get a root action
  /// which will be disabled.
  TingyunRootAction enterAction(String name, {int type = 0});

  // /// Report an error with [errorName] and [errorCode] directly without any action
  // void reportError(String errorName, int errorCode, {Platform platform});
  void reportError(String message, String stackTrace, {Map<String, dynamic>? props});

  /// Report an error which contains a [stacktrace] and will therefor be reported as a real
  /// crash. The error includes [errorName] and [reason] as well.
  // Future<void> reportCrash(String errorName, String reason, String stacktrace, {int type = 1});

  /// Closes the session in the next possible moment.
  void endSession();

  /// Will attach a [user] to the session so you can identify the session later on.
  void setUserIdentifier(String user);

  void leaveBreadcrumb(String breadCrumb);
  void setUserCrashMessage(String key, String value);
  void onEvent(String eventId, {String eventTag = "", Map<String, dynamic> props});

  // /// Sets the GPS location of the session with [latitude] and [longitude].
  // void setGPSLocation(double latitude, double longitude, {Platform platform});

  // /// Returns the current value for crash reporting of the NATIVE agent.
  // Future<bool> isCrashReportingOptedIn({Platform platform});

  // /// Sets the [crashReporting] value for the NATIVE agent.
  // void setCrashReportingOptedIn(bool crashReporting, {Platform platform});

  // /// Returns the [DataCollectionLevel] for the NATIVE agent. Only valid when user opt in mode is used.
  // /// If not used, it will return User mode per default.
  // Future<DataCollectionLevel> getDataCollectionLevel({Platform platform});

  // /// Set the data collection level for the NATIVE agent. Only valid when user opt in mode is used.
  // void setDataCollectionLevel(DataCollectionLevel dataCollectionLevel,
  //     {Platform platform});

  /// Send events if necessary.
  void flushEvents();

  int getPageLoadingDuration();

  int getNetworkDelay();

  Future<TYWebRequestTiming> createWebRequest(String url);

  void setCustomOnResumeEndIns();

  void startPage({String pageName =""});

  void endPage();

}